(** val negb : bool -> bool **)

let negb = function
| true -> false
| false -> true

(** val app : 'a1 list -> 'a1 list -> 'a1 list **)

let rec app l m =
  match l with
  | [] -> m
  | a :: l1 -> a :: (app l1 m)

(** val plus : int -> int -> int **)

let rec plus = (+)

(** val mult : int -> int -> int **)

let rec mult = ( * )

(** val minus : int -> int -> int **)

let rec minus = fun n m -> Pervasives.max 0 (n-m)

(** val nth : int -> 'a1 list -> 'a1 -> 'a1 **)

let rec nth n l default =
  (fun fO fS n -> if n=0 then fO () else fS (n-1))
    (fun _ ->
    match l with
    | [] -> default
    | x :: l' -> x)
    (fun m ->
    match l with
    | [] -> default
    | x :: t -> nth m t default)
    n

(** val init_row_list : int -> (int -> 'a1) -> 'a1 list **)

let rec init_row_list n f =
  (fun fO fS n -> if n=0 then fO () else fS (n-1))
    (fun _ ->
    [])
    (fun n' ->
    app (init_row_list n' f) ((f n') :: []))
    n

(** val init_matrix_list : int -> int -> (int -> int -> 'a1) -> 'a1 list **)

let rec init_matrix_list m n f =
  (fun fO fS n -> if n=0 then fO () else fS (n-1))
    (fun _ ->
    [])
    (fun m' ->
    app (init_matrix_list m' n f) (init_row_list n (f m')))
    m

(** val elem : 'a1 -> int -> int -> int -> 'a1 list -> 'a1 **)

let elem zero n i j ul =
  nth (plus (mult n i) j) ul zero

(** val add_matrix_list :
    ('a1 -> 'a1 -> 'a1) -> 'a1 -> int -> int -> 'a1 list -> 'a1 list -> 'a1
    list **)

let add_matrix_list add zero m n u v =
  init_matrix_list m n (fun i j ->
    add (elem zero m i j u) (elem zero m i j v))

(** val bilinear_list :
    ('a1 -> 'a1 -> 'a1) -> ('a1 -> 'a1 -> 'a1) -> 'a1 -> int -> int -> 'a1
    list -> int -> int -> 'a1 list -> int -> 'a1 **)

let rec bilinear_list add mul zero m i ul n j vl k =
  (fun fO fS n -> if n=0 then fO () else fS (n-1))
    (fun _ ->
    zero)
    (fun k' ->
    add (bilinear_list add mul zero m i ul n j vl k')
      (mul (elem zero m i k' ul) (elem zero n k' j vl)))
    k

(** val mul_matrix_list :
    ('a1 -> 'a1 -> 'a1) -> ('a1 -> 'a1 -> 'a1) -> 'a1 -> int -> int -> 'a1
    list -> int -> int -> 'a1 list -> 'a1 list **)

let mul_matrix_list add mul zero m1 n1 ul m2 n2 vl =
  init_matrix_list m1 n2 (fun i j ->
    bilinear_list add mul zero n1 i ul n2 j vl m2)

(** val beq_nat : int -> int -> bool **)

let rec beq_nat n m =
  (fun fO fS n -> if n=0 then fO () else fS (n-1))
    (fun _ ->
    (fun fO fS n -> if n=0 then fO () else fS (n-1))
      (fun _ ->
      true)
      (fun m' ->
      false)
      m)
    (fun n' ->
    (fun fO fS n -> if n=0 then fO () else fS (n-1))
      (fun _ ->
      false)
      (fun m' ->
      beq_nat n' m')
      m)
    n

(** val ble_nat : int -> int -> bool **)

let rec ble_nat n m =
  (fun fO fS n -> if n=0 then fO () else fS (n-1))
    (fun _ ->
    true)
    (fun n' ->
    (fun fO fS n -> if n=0 then fO () else fS (n-1))
      (fun _ ->
      false)
      (fun m' ->
      ble_nat n' m')
      m)
    n

(** val blt_nat : int -> int -> bool **)

let blt_nat n m =
  (&&) (ble_nat n m) (negb (beq_nat n m))

(** val cofactor_elem :
    'a1 -> int -> int -> int -> 'a1 list -> int -> int -> 'a1 **)

let cofactor_elem zero n i' j' ul i j =
  if blt_nat i i'
  then if blt_nat j j'
       then elem zero n i j ul
       else elem zero n i (plus j (Pervasives.succ 0)) ul
  else if blt_nat j j'
       then elem zero n (plus i (Pervasives.succ 0)) j ul
       else elem zero n (plus i (Pervasives.succ 0))
              (plus j (Pervasives.succ 0)) ul

(** val cofactor_matrix :
    'a1 -> int -> int -> int -> int -> 'a1 list -> 'a1 list **)

let cofactor_matrix zero m n i j u =
  init_matrix_list (minus m (Pervasives.succ 0))
    (minus n (Pervasives.succ 0)) (cofactor_elem zero m i j u)

(** val mod2 : int -> int **)

let rec mod2 n =
  (fun fO fS n -> if n=0 then fO () else fS (n-1))
    (fun _ ->
    0)
    (fun n0 ->
    (fun fO fS n -> if n=0 then fO () else fS (n-1))
      (fun _ -> Pervasives.succ
      0)
      (fun n' ->
      mod2 n')
      n0)
    n

(** val list_sign_sum_F :
    ('a1 -> 'a1 -> 'a1) -> ('a1 -> 'a1 -> 'a1) -> 'a1 -> int -> 'a1 -> 'a1
    list -> 'a1 **)

let rec list_sign_sum_F add sub zero n ret l =
  (fun fO fS n -> if n=0 then fO () else fS (n-1))
    (fun _ ->
    ret)
    (fun n' ->
    (fun fO fS n -> if n=0 then fO () else fS (n-1))
      (fun _ ->
      list_sign_sum_F add sub zero n' (add ret (nth n' l zero)) l)
      (fun n0 ->
      (fun fO fS n -> if n=0 then fO () else fS (n-1))
        (fun _ ->
        list_sign_sum_F add sub zero n' (sub ret (nth n' l zero)) l)
        (fun n1 ->
        list_sign_sum_F add sub zero n' ret l)
        n0)
      (mod2 n'))
    n

(** val determinant :
    ('a1 -> 'a1 -> 'a1) -> ('a1 -> 'a1 -> 'a1) -> ('a1 -> 'a1 -> 'a1) -> 'a1
    -> int -> 'a1 list -> 'a1 **)

let rec determinant add sub mul zero n u =
  (fun fO fS n -> if n=0 then fO () else fS (n-1))
    (fun _ ->
    zero)
    (fun n' ->
    (fun fO fS n -> if n=0 then fO () else fS (n-1))
      (fun _ ->
      elem zero (Pervasives.succ 0) 0 0 u)
      (fun n0 ->
      list_sign_sum_F add sub zero n zero
        (init_row_list n (fun k ->
          mul (elem zero n 0 k u)
            (determinant add sub mul zero n'
              (cofactor_matrix zero n n 0 k u)))))
      n')
    n

