(* 余因子行列 *)
let cofactor_row mat n idx =
  let n_row = if idx < n then idx else idx + 1 in
  let n_col = Array.length mat.(n_row) in
  Array.init (n_col-1) (fun k -> mat.(n_row).(k+1))

let cofactor mat n =
  let n_row = Array.length mat in
  Array.init (n_row-1) (cofactor_row mat n)

let sign i = if i mod 2 = 0 then 1. else -1.

(* 2次の行列式 *)
let determinant_2 mat =
  mat.(0).(0) *. mat.(1).(1) -. mat.(1).(0) *. mat.(0).(1)

(* 行列式 *)
let rec determinant mat =
  let n_row = Array.length mat in
  if n_row = 1 then mat.(0).(0)
  else if n_row = 2 then determinant_2 mat
  else
    let cofactors =
      Array.init n_row (fun k -> (cofactor mat k)) in
    let dets = Array.map determinant cofactors in
    let sums =
      Array.init n_row (fun k -> (sign k) *. mat.(k).(0) *. dets.(k)) in
    Array.fold_left (+.) 0. sums

let () =
  let a = [|
    [|1.;2.;3.;2.|];
    [|0.;1.;1.;-1.|];
    [|1.;0.;-1.;1.|];
    [|0.;2.;-2.;5.|]
  |] in
  let r = determinant a in
  print_endline (string_of_float r)
