
(* 遅延評価による Euler 法の計算  *)
open Lazy

exception Empty_stream

type 'a stream = Nils | Cons of 'a * 'a stream Lazy.t

let ( +& ) (x1, v1) (x2, v2) =
  (x1 +. x2, v1 +. v2)

let ( *< ) c (x, v) =
  (c *. x, c *. v)
  
let eom_euler h f (x, v) =
  (x, v) +& (h *< f (x, v))
 
let rec lazy_eom_euler h f (x0, v0) = Cons ((x0, v0), lazy (lazy_eom_euler h f (eom_euler h f (x0, v0))))

let rec stream_ref s n =
  match s with
  | Nils -> raise Empty_stream
  | Cons (x, _) when n = 1 -> x
  | Cons (_, tail) -> stream_ref (force tail) (n-1)

let tpi = 8. *. atan 1.
let f (x, v) = (v, -.(tpi*.tpi) *. x)
let s = lazy_eom_euler 0.0001 f (0., 1.)
let print_vector_fst (x, v) = print_float x
let rec dump_stream n idx =
  print_vector_fst (stream_ref s (idx+1));
  print_endline "";
  if (idx + 1) > n then ()
  else dump_stream n (idx+1)

let () = dump_stream 10000 0
