
Require Import Arith.
Require Import List.
Open Scope list_scope.

Record particle := { x : nat; y : nat }.


Definition beq_particle (p p' : particle) : bool :=
  match beq_nat (x p) (x p'), beq_nat (y p) (y p') with
    | true, true => true
    | _, _ => false
  end.

Fixpoint make_workpiece_row (z x0 w : nat) : list particle :=
  match w with
    | O => nil
    | S w' => (make_workpiece_row z x0 w') ++ (Build_particle (x0+w') z :: nil)
  end.

Fixpoint make_workpiece (z0 h x0 w : nat) : list particle :=
  match h with
    | O => nil
    | S h' => (make_workpiece z0 h' x0 w) ++ (make_workpiece_row (z0+h') x0 w)
  end.

Compute make_workpiece 0 10 0 10.

Fixpoint inner (pl : list particle) (p : particle) : bool :=
  match pl with
    | nil => false
    | x :: xs => match beq_particle x p with
                   | true => true
                   | false => inner xs p
                 end
  end.

Definition outer (pl : list particle) (p : particle) : bool := negb (inner pl p).

Compute inner (Build_particle 1 1 :: Build_particle 1 2 :: nil) (Build_particle 1 1).
Compute inner (Build_particle 1 1 :: Build_particle 1 2 :: nil) (Build_particle 2 2).

Definition remove_particle (pl : list particle) (p : particle) : list particle :=
  filter (fun p' => negb (beq_particle p p')) pl.

Fixpoint remove_particle_list (pl pls : list particle) : list particle :=
  match pls with
    | nil => pl
    | px :: pxs
      => remove_particle_list (remove_particle pl px) pxs
  end.

Compute remove_particle (Build_particle 1 1 :: Build_particle 1 2 :: Build_particle 1 3 :: nil) (Build_particle 1 1).

Lemma beq_nat_true : forall (n : nat), beq_nat n n = true.
Proof.
  intros.
  induction n.
  simpl; reflexivity.
  simpl; exact IHn.
Qed.

Lemma beq_particle_true : forall (n m : nat), beq_particle {| x:=n; y:=m |} {| x:=n; y:=m |} = true.
Proof.
  intros.
  unfold beq_particle.
  simpl.
  repeat rewrite beq_nat_true.
  reflexivity.
Qed.
  
Lemma remove_particle_left : forall (n m1 m2 : nat), remove_particle (Build_particle n m1 :: Build_particle n m2 :: nil) (Build_particle n m2) = remove_particle (Build_particle n m1 :: nil) (Build_particle n m2).
Proof.
  intros.
  unfold remove_particle.
  simpl.
  rewrite beq_particle_true.
  simpl.
  reflexivity.
Qed.

Lemma remove_each_left : forall (pl : list particle) (n m : nat), remove_particle (Build_particle n m :: pl) (Build_particle n m) = remove_particle pl (Build_particle n m).
Proof.
  intros.
  unfold remove_particle.
  simpl.
  rewrite beq_particle_true.
  simpl.
  reflexivity.
Qed.
